;;
;; ZixSaw/theme.jl       (ZixSaw 0.0.2 for sawfish 0.30.3)
;;
;; by Zinx Verituse <zinx@linuxfreak.com>
;;
;; Please excuse my lisp :)
;;
;; Some code from blackDome was referenced...
;;

(let*
    (
	(font (get-font "-*-lucida-medium-r-normal-*-*-100-*-*-p-*-iso8859-1"))
	(font-colors (list "grey50" "rgb:c/c/c"))

	(title-width
	  (lambda (w)
	    (max 64
	      (min (- (car (window-dimensions w)) 16) (text-width (window-name w) font))
	    )
	  )
	)

	(update-title-width
	  (lambda (w)
	    (if (eq (window-get w 'current-frame-style) 'ZixSaw)
	      (rebuild-frame w)
	    )
	  )
	)

	(titlebar-left (make-image "zix-titlebar-left.png"))
        (titlebar-mid (make-image "zix-titlebar-mid.png"))
	(titlebar-right (make-image "zix-titlebar-right.png"))

	(saw-image (make-image "zix-saw.png"))

	(sidebar-full (make-image "zix-sidebar-full.png"))

	(button-close (make-image "zix-button-close.png"))
	(button-minimize (make-image "zix-button-minimize.png"))
	(button-menu (make-image "zix-button-menu.png"))

	(resize-bump (make-image "zix-resize-bump.png"))

	;; The Saw...
        (saw
        `(
	  ((background . ,saw-image)
	   (left-edge . -23)
	   (top-edge . -22)
	   (class . title))
         )
	)

	;; Titlebar (buttons on side)...
        (titlebar-corner
        `(
          ((background . ,titlebar-left)
           (top-edge . -19)
	   (left-edge . -12)
	   (class . title))
	  ((background . ,titlebar-mid)
	   (foreground . ,font-colors)
	   (font . ,font)
	   (text . ,window-name)
	   (x-justify . left)
	   (y-justify . center)
	   (top-edge . -19)
	   (left-edge . 4)
	   (width . ,(lambda (w) (+ (title-width w) 4)))
	   (class . title))
	  ((background . ,titlebar-right)
	   (top-edge . -19)
	   (left-edge . ,(lambda (w) (+ (title-width w) 4)))
	   (class . title))

	        ;; Bar on the side...

	  ((background . ,sidebar-full)
	   (top-edge . -14)
	   (left-edge . -19)
	   (height . 74))

		;; Buttons on the bar...

	  ((background . ,button-close)
	   (top-edge . 5)
	   (left-edge . -15)
	   (class . close-button))
	  ((background . ,button-minimize)
	   (top-edge . 18)
	   (left-edge . -15)
	   (class . iconify-button))
	  ((background . ,button-menu)
	   (top-edge . 30)
	   (left-edge . -15)
	   (class . menu-button))
         )
        )


        ;; Titlebar (buttons after text)...
	(titlebar-top
	`(
          ((background . ,titlebar-left)
           (top-edge . -19)
	   (left-edge . -12)
	   (class . title))
	  ((background . ,titlebar-mid)
	   (foreground . ,font-colors)
	   (font . ,font)
	   (text . ,window-name)
	   (x-justify . left)
	   (y-justify . center)
	   (top-edge . -19)
	   (left-edge . 4)
	   (width . ,(lambda (w) (+ (title-width w) 40)))
	   (class . title))
	  ((background . ,titlebar-right)
	   (top-edge . -19)
	   (left-edge . ,(lambda (w) (+ (title-width w) 40)))
	   (class . title))

		;; Buttons on the bar (titlebar this time)...

	  ((background . ,button-close)
	   (top-edge . -15)
	   (left-edge . ,(lambda (w) (+ (title-width w) 4)))
	   (class . close-button))
	  ((background . ,button-minimize)
	   (top-edge . -13)
	   (left-edge . ,(lambda (w) (+ (title-width w) 16)))
	   (class . iconify-button))
	  ((background . ,button-menu)
	   (top-edge . -15)
	   (left-edge . ,(lambda (w) (+ (title-width w) 28)))
	   (class . menu-button))
         )
        )

	;; Thin Cyan borders
	(borders
	`(
	  ((background . "rgb:24/4b/62")
	   (top-edge . -1)
	   (left-edge . 0)
	   (right-edge . 0)
	   (height . 1))
	  ((background . "rgb:24/4b/62")
	   (bottom-edge . -1)
	   (left-edge . 0)
	   (right-edge . 0)
	   (height . 1))
	  ((background . "rgb:24/4b/62")
	   (left-edge . -1)
	   (top-edge . 0)
	   (bottom-edge . 0)
	   (width . 1))
	  ((background . "rgb:24/4b/62")
	   (right-edge . -1)
	   (top-edge . 0)
	   (bottom-edge . 0)
	   (width . 1))
         )
        )

	;; Resize bump
        (resizer
        `(
	  ((background . ,resize-bump)
	   (right-edge . -4)
	   (bottom-edge . -4)
	   (class . bottom-right-corner))
         )
        )

        (frame `(,@borders ,@titlebar-corner ,@resizer ,@saw))
        (transient-frame `(,@borders ,@titlebar-corner ,@resizer))
        (shaped-frame `(,@titlebar-corner ,@resizer ,@saw))
        (shaped-transient-frame `(,@titlebar-top ,@resizer))
        (shaded-frame `(,@titlebar-top ,@saw))
        (shaded-transient-frame `(,@titlebar-top))
        (unknown-frame `(,@titlebar-top ,@saw))
    )

    (add-frame-style 'ZixSaw
      (lambda (w type)
        (cond ((eq type 'default) frame)
              ((eq type 'unframed) nil-frame)
              ((eq type 'transient) transient-frame)
              ((eq type 'shaded) shaded-frame)
              ((eq type 'shaped) shaped-frame)
              ((eq type 'shaded-transient) shaded-transient-frame)
              ((eq type 'shaped-transient) shaped-transient-frame)
              (t unknown-frame)
        )
      )
    )

    (call-after-property-changed 'WM_NAME update-title-width)
)

